// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/Scene.hpp"
#include "scene/components/display/Camera.hpp"
#include "prefabs/SimpleSpinningCubePrefab.hpp"
#include "games/vn/components/VNManager.hpp"
#include "games/vn/events/VNDummyEvent.hpp"
#include "games/vn/events/VNPositionEvent.hpp"

namespace Dawn {
  class HelloWorldScene : public Scene {
    protected:
      Camera *camera;
      UICanvas *canvas;

      void stage() override {
        canvas = UICanvas::create(this);
        
        camera = Camera::create(this);
        camera->fov = 0.436332f;
        camera->transform->lookAt(glm::vec3(10, 10, 10), glm::vec3(0, 0, 0));

        auto cube = SimpleSpinningCubePrefab::create(this);

        auto vnItem = this->createSceneItem();
        auto vnManager = vnItem->addComponent<VNManager>();

        auto eventTest = vnManager->createEvent<VNDummyEvent>();

        auto positionEvent = vnManager->createEvent<VNPositionEvent>();
        positionEvent->x = 2.0f;
        positionEvent->item = cube;
        positionEvent->duration = 3.0f;

        eventTest
          ->then(positionEvent)
          ->then(vnManager->createEvent<VNDummyEvent>())
          ->then(vnManager->createEvent<VNDummyEvent>())
        ;
        vnManager->setEvent(eventTest);
      }
      
      std::vector<Asset*> getRequiredAssets() override {
        auto assMan = &this->game->assetManager;
        std::vector<Asset*> assets;
        return assets;
      }

    public:
      HelloWorldScene(DawnGame *game) : Scene(game) {}
  };
}